# $Id: /mirror/gungho/lib/Gungho/Plugin.pm 6450 2007-04-10T01:52:17.416998Z lestrrat  $
#
# Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
# All rights reserved.

package Gungho::Plugin;
use strict;
use warnings;
use base qw(Gungho::Base);

1;

__END__

=head1 NAME

Gungho::Plugin - Gungho Plugin Base Class

=cut
